from Compatibility import *

class array(object):
    """Wraps a function or a nested list or tuple as a multidimensional array
    Access to a cell of an ArrayWrapper object, e.g. "aw[a,b,c]" will be
    mapped to a function call "f(a,b,c)" or to a nested sequence "l[a][b][c]"
    respectively.

    The ArrayWrapper class is syntactical sugar to provide similar access to
    pay functions or lists as to Numeric or nummarray arrays.

    Warning: This wrapper does not do any error checking at all!
    """
    
    def __init__(self, obj):
        """obj - function or tuple of list
        """
        if callable(obj):
            self.obj = None
            self.function = obj
        elif hasattr(obj, "__getitem__"):
            self.obj = obj
            self.function = self.__listwrapper
        else:
            raise TypeError, "ArrayWrapper only supports callables, lists "+\
                             "or tuples, but not %s!" % type(obj)
                             
    def __listwrapper(self, *key):
        value = self.obj
        for i in key:  value = value[i]
        return value
        
    def __getitem__(self, key):
        return self.function(*key)

    def __getstate__(self):
        if self.function != self.__listwrapper:
            raise TypeError, "Can't prepare an Array Wrapper "\
                             "of a function for pickling!"
        else:
            dict = self.__dict__.copy()
            dict["function"] = None
            return dict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.function = self.__listwrapper        
    
    def tolist(self):
        """Returns the object as list. Not possible for wrapped functions!"""
        if self.obj:
            return self.obj
        else:  raise TypeError, "Cannot convert wraped function into a list!"
